-------------------------------------------------------------------------------
-- This file is part of libsparkcrypto.
--
-- Copyright (C) 2010, Alexander Senier
-- Copyright (C) 2010, secunet Security Networks AG
-- All rights reserved.
--
-- Redistribution  and  use  in  source  and  binary  forms,  with  or  without
-- modification, are permitted provided that the following conditions are met:
--
--    * Redistributions of source code must retain the above copyright notice,
--      this list of conditions and the following disclaimer.
--
--    * Redistributions in binary form must reproduce the above copyright
--      notice, this list of conditions and the following disclaimer in the
--      documentation and/or other materials provided with the distribution.
--
--    * Neither the name of the  nor the names of its contributors may be used
--      to endorse or promote products derived from this software without
--      specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE  COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY  EXPRESS OR IMPLIED WARRANTIES,  INCLUDING, BUT NOT LIMITED  TO, THE
-- IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR  A PARTICULAR PURPOSE
-- ARE  DISCLAIMED. IN  NO EVENT  SHALL  THE COPYRIGHT  HOLDER OR  CONTRIBUTORS
-- BE  LIABLE FOR  ANY  DIRECT, INDIRECT,  INCIDENTAL,  SPECIAL, EXEMPLARY,  OR
-- CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT  LIMITED  TO,  PROCUREMENT  OF
-- SUBSTITUTE GOODS  OR SERVICES; LOSS  OF USE,  DATA, OR PROFITS;  OR BUSINESS
-- INTERRUPTION)  HOWEVER CAUSED  AND ON  ANY THEORY  OF LIABILITY,  WHETHER IN
-- CONTRACT,  STRICT LIABILITY,  OR  TORT (INCLUDING  NEGLIGENCE OR  OTHERWISE)
-- ARISING IN ANY WAY  OUT OF THE USE OF THIS SOFTWARE, EVEN  IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
-------------------------------------------------------------------------------

with LSC.Types;
use type LSC.Types.Word64;
use type LSC.Types.Index;
--# inherit
--#    LSC.Types,
--#    LSC.Byteorder64;

-------------------------------------------------------------------------------
-- Cryptographic padding for arrays of 64-bit words
-------------------------------------------------------------------------------
package LSC.Pad64
is

   -- Terminate a Word64 array
   --
   -- The array @Block@ is terminated by setting the bit at (@Length@ + 1) to 1
   -- and all following bits to 0.
   --
   procedure Block_Terminate
     (Block  : in out Types.Word64_Array_Type;
      Length : in     Types.Word64);
   --
   -- <strong> NOTE: The postcondition currently does not completely express
   --          the intended behaviour of the operation! </strong>
   --
   --# derives Block from *,
   --#                    Length;
   --# pre
   --#    Types.Index'First + Types.Index (Length / 64) in Block'Range;
   --# post
   --#    (for all I in Types.Index range
   --#        Types.Index'First + Types.Index (Length / 64) + 1 .. Block'Last => (Block (I) = 0));

end LSC.Pad64;
